@@ -4,6 +4,7 @@
 #include <string.h>
 
 #include <orbit/orbit.h>
+#include <orbit/orb-core/orb-types.h>
 
 #include "../orbit-init.h"
 #include "../poa/orbit-poa-export.h"
@@ -22,6 +23,13 @@ extern const ORBit_option orbit_supported_options[];
 OrbitDebugFlags _orbit_debug_flags = ORBIT_DEBUG_NONE;
 #endif
 
+#ifdef G_OS_WIN32
+/* Silence gcc warning about no prototype */
+BOOL WINAPI DllMain (HINSTANCE hinstDLL,
+		     DWORD     fdwReason,
+		     LPVOID    lpvReserved);
+#endif
+
 /*
  * Command line option handling.
  */
@@ -41,6 +49,7 @@ static gboolean     orbit_use_irda           = FALSE;
 static gboolean     orbit_use_ssl            = FALSE;
 static gboolean     orbit_use_genuid_simple  = FALSE;
 static gboolean     orbit_local_only         = FALSE;
+static char        *orbit_net_id             = NULL;
 static gboolean     orbit_use_http_iors      = FALSE;
 static char        *orbit_ipsock             = NULL;
 static char        *orbit_ipname             = NULL;
@@ -65,6 +74,32 @@ ORBit_ORB_start_servers (CORBA_ORB orb)
 	if (orbit_local_only)
 		create_options |= LINK_CONNECTION_LOCAL_ONLY;
 
+	if (orbit_local_only || (orbit_use_usocks && !(orbit_use_ipv4 || orbit_use_ipv6 || orbit_use_irda || orbit_use_ssl)))
+			link_use_local_hostname (LINK_NET_ID_IS_LOCAL);
+	else {
+		do {
+			if (!orbit_net_id)
+				break;
+
+			if (!strcmp(orbit_net_id, "local")) {
+				link_use_local_hostname (LINK_NET_ID_IS_LOCAL);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "short")) {
+				link_use_local_hostname (LINK_NET_ID_IS_SHORT_HOSTNAME);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "fqdn")) {
+				link_use_local_hostname (LINK_NET_ID_IS_FQDN);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "ipaddr")) {
+				link_use_local_hostname (LINK_NET_ID_IS_IPADDR);
+				break;
+			}
+		} while (0);
+	}
+		    
 	for (info = link_protocol_all (); info->name; info++) {
 		GIOPServer           *server;
 
@@ -322,13 +357,16 @@ ORBit_initial_references_by_user (CORBA_ORB          orb,
 			g_warning ("Option ORBInitRef has invalid object reference: %s=%s",  
 				   tuple->key, tuple->value);
 			CORBA_exception_free (ev);
-		}  else if (ORBit_initial_reference_protected_id(tuple->key)) {
-                        g_warning ("Option ORBInitRef permission denied: %s=%s",
-
-                                   tuple->key, tuple->value);
-                } else {
-                        ORBit_set_initial_reference (orb, tuple->key, objref);
-                }
+		} else {
+			if (ORBit_initial_reference_protected_id(tuple->key)) {
+				g_warning ("Option ORBInitRef permission denied: %s=%s",
+					   tuple->key, tuple->value);
+			} else {
+				ORBit_set_initial_reference (orb, tuple->key, objref);
+			}
+			
+			ORBit_RootObject_release (objref);
+		}
 	}
 
 }
@@ -397,6 +435,13 @@ CORBA_ORB_init (int *argc, char **argv,
 	retval->default_giop_version = GIOP_LATEST;
 
 	retval->adaptors = g_ptr_array_new ();
+
+	/* init the forward bind hashtable*/
+	retval->forw_binds = g_hash_table_new_full (
+		g_str_hash, g_str_equal,
+		g_free,
+		NULL); 
+
 	ORBit_init_internals (retval, ev);
 	/* FIXME, handle exceptions */ 
 
@@ -566,13 +611,13 @@ CORBA_ORB_create_list (CORBA_ORB          obj,
 		       CORBA_NVList      *new_list,
 		       CORBA_Environment *ev)
 {
-	CORBA_NVList new;
+	CORBA_NVList nvlist;
 
-	new = g_new0 (struct CORBA_NVList_type, 1);
+	nvlist = g_new0 (struct CORBA_NVList_type, 1);
 
-	new->list = g_array_new (FALSE, TRUE, sizeof (CORBA_NamedValue));
+	nvlist->list = g_array_new (FALSE, TRUE, sizeof (CORBA_NamedValue));
 
-	*new_list = new;
+	*new_list = nvlist;
 }
 
 void
@@ -1235,8 +1280,16 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 		}
 	}
 
+	/* destroy the forward bind hashtable*/
+	g_hash_table_destroy (orb->forw_binds);
+	
 	orb->life_flags |= ORBit_LifeF_Destroyed;
 
+	if (orb->lock) {
+		g_mutex_free (orb->lock);
+		orb->lock = NULL;
+	}
+
 	ORBit_RootObject_release (orb);
 
 	/* At this stage there should be 1 ref left in the system -
@@ -1284,13 +1337,15 @@ ORBit_set_initial_reference (CORBA_ORB  orb,
 			     gchar     *identifier,
 			     gpointer   objref)
 {
+	CORBA_Object old_objref;
+
 	if (!orb->initial_refs)
 		orb->initial_refs = g_hash_table_new (g_str_hash, g_str_equal);
 
-	if (g_hash_table_lookup (orb->initial_refs, identifier))
-               /* FIXME, release object before removing from hash-table,
-                   Frank 2003/06/22 */
+	if ((old_objref = g_hash_table_lookup (orb->initial_refs, identifier))) {
+		ORBit_RootObject_release (old_objref);
 		g_hash_table_remove (orb->initial_refs, identifier);
+	}
 
 	g_hash_table_insert (orb->initial_refs,
 			     identifier,
@@ -1303,7 +1358,13 @@ ORBit_ORB_forw_bind (CORBA_ORB                   orb,
 		     CORBA_Object                obj,
 		     CORBA_Environment          *ev)
 {
-	g_warning ("ORBit_ORB_forw_bind NYI");
+	
+	if (obj)
+		g_hash_table_insert (orb->forw_binds, objkey->_buffer, obj);
+	else {
+		g_hash_table_remove(orb->forw_binds, objkey->_buffer);
+	}
+	
 }
 
 gboolean
@@ -1331,6 +1392,7 @@ const ORBit_option orbit_supported_options[] = {
  	{ "ORBIIOPIPSock",      ORBIT_OPTION_STRING,  &orbit_ipsock },
 	{ "ORBInitialMsgLimit", ORBIT_OPTION_INT,     &orbit_initial_recv_limit },
 	{ "ORBLocalOnly",       ORBIT_OPTION_BOOLEAN, &orbit_local_only },
+ 	{ "ORBNetID",           ORBIT_OPTION_STRING,  &orbit_net_id },
 	/* warning: this option is a security risk unless used with LocalOnly */
 	{ "ORBIIOPIPv4",        ORBIT_OPTION_BOOLEAN, &orbit_use_ipv4 },
 	{ "ORBIIOPIPv6",        ORBIT_OPTION_BOOLEAN, &orbit_use_ipv6 },
@@ -1347,3 +1409,69 @@ const ORBit_option orbit_supported_options[] = {
 	{ "ORBCorbaloc",        ORBIT_OPTION_BOOLEAN, &orbit_use_corbaloc},
 	{ NULL,                 0,                    NULL },
 };
+
+#ifdef G_OS_WIN32
+
+const gchar *ORBit_win32_typelib_dir;
+const gchar *ORBit_win32_system_rcfile;
+
+/* DllMain function needed to fetch the DLL name and deduce the
+ * installation directory from that, and then form the pathnames for
+ * the typelib directory and system orbitrc file.
+ */
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+  wchar_t wcbfr[1000];
+  char cpbfr[1000];
+  char *dll_name = NULL;
+  gchar *prefix;
+  
+  switch (fdwReason) {
+  case DLL_PROCESS_ATTACH:
+	  /* GLib 2.6 uses UTF-8 file names */
+	  if (GetVersion () < 0x80000000) {
+		  /* NT-based Windows has wide char API */
+		  if (GetModuleFileNameW ((HMODULE) hinstDLL,
+					  wcbfr, G_N_ELEMENTS (wcbfr)))
+			  dll_name = g_utf16_to_utf8 (wcbfr, -1,
+						      NULL, NULL, NULL);
+	  } else {
+		  /* Win9x, yecch */
+		  if (GetModuleFileNameA ((HMODULE) hinstDLL,
+					  cpbfr, G_N_ELEMENTS (cpbfr)))
+			  dll_name = g_locale_to_utf8 (cpbfr, -1,
+						       NULL, NULL, NULL);
+	  }
+
+	  if (dll_name) {
+		  gchar *p = strrchr (dll_name, '\\');
+		  
+		  if (p != NULL)
+			  *p = '\0';
+
+		  p = strrchr (dll_name, '\\');
+		  if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0))
+			  *p = '\0';
+		  
+		  prefix = dll_name;
+	  } else {
+		  prefix = g_strdup ("");
+	  }
+
+	  ORBit_win32_typelib_dir = g_strconcat (prefix,
+						 "\\lib\\orbit-2.0",
+						 NULL);
+	  ORBit_win32_system_rcfile = g_strconcat (prefix,
+						   "\\etc\\orbitrc",
+						   NULL);
+	  g_free (prefix);
+	  break;
+  }
+
+  return TRUE;
+}
+
+#endif
